/*
 *
 * RepRap, The Replicating Rapid Prototyper Project
 *
 * http://reprap.org/
 *
 * RepRap is copyright (C) 2005-6 University of Bath, the RepRap
 * researchers (see the project's People webpage), and other contributors.
 *
 * RepRap is free; you can redistribute it and/or modify it under the
 * terms of the GNU Library General Public Licence as published by the
 * Free Software Foundation; either version 2 of the Licence, or (at your
 * option) any later version.
 *
 * RepRap is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
 * Licence for more details.
 *
 * For this purpose the words "software" and "library" in the GNU Library
 * General Public Licence are taken to mean any and all computer programs
 * computer files data results documents and other copyright information
 * available from the RepRap project.
 *
 * You should have received a copy of the GNU Library General Public
 * Licence along with RepRap (in reports, it will be one of the
 * appendices, for example); if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA, or see
 *
 * http://www.gnu.org/
 *
 */

/*
 * This program fades a LED on and off once a second.  It's intended to
 * be used to verify that your PIC programmer is working and your PIC is 
 * actually executing instructions when you can't establish serial 
 * communication.
 */

#include <pic14.h>

typedef unsigned int config;
config at 0x2007 __CONFIG = _CP_OFF &
 _WDT_OFF &
 _BODEN_OFF &
 _PWRTE_ON &
 _INTRC_OSC_NOCLKOUT &
 _MCLRE_OFF &
 _LVP_OFF;

#define LED PORTA4

// 0 .. 38 = 39 brightness levels gives us 100Hz PWM, which is about the
// slowest we can go without too much flicker
#define DUTY_CYCLE_MAX 38
// 39 steps * 256us = 9984us, which is close enough to 100Hz for me.
#define PWM_HZ 100 

void main() {
  unsigned char duty_cycle = 0; // the current brightness level
  unsigned char step = 0;       // counts from 0 .. PWM_HZ/2 .. 0
  unsigned char direction = 0;  // 0 = brighten, 1 = dim
  unsigned char pwm_period_counter = 0;

  PORTA = 0x10; // RA4 is high to turn of LED
  CMCON = 0x07; // disable comparators
  TRISA = 0x00;

  PORTB = 0x00;
  TRISB = 0x02; // RB2 is RX

  
  // initialize TIMER0 to run as fast as it can, so it overflows every 256us.
  OPTION_REG = BIN(11001000);
  TMR0 = 0;
  T0IE = 0;
  T0IF = 0;

  while (1) {
    while (!T0IF);
    // timer0 has overflowed, so we've advanced another 1/DUTY_CYCLE_MAX
    // of our PWM duty cycle.  Check to see if we need to change
    // output
    if (pwm_period_counter == 0 && duty_cycle > 0) 
      LED = 0;
    else if (pwm_period_counter == duty_cycle) 
      LED = 1;

    pwm_period_counter++;
    if (pwm_period_counter > DUTY_CYCLE_MAX) {
      pwm_period_counter = 0;
      // calculate the brightness for the next PWM period
      // Fading the LED on and off one step per PWM period only 
      // takes 39 * 2 steps (780ms), so we add a period of dark before and
      // after the fade to make it add up to a second
      if (direction == 0) {
        step++;
        if (step == (PWM_HZ/2)) {
          direction = 1;
          duty_cycle = DUTY_CYCLE_MAX - 1;
        } else if (step > (PWM_HZ/2 - DUTY_CYCLE_MAX)) {
          duty_cycle++;
        }
      } else /* direction == 1 */ {
        step--;
        if (step == 0)
          direction = 0;
        else if (duty_cycle > 0)
          duty_cycle--;
      }
    }
    T0IF = 0;
  }
}

