/*
 *
 * RepRap, The Replicating Rapid Prototyper Project
 *
 * http://reprap.org/
 *
 * RepRap is copyright (C) 2005-6 University of Bath, the RepRap
 * researchers (see the project's People webpage), and other contributors.
 *
 * RepRap is free; you can redistribute it and/or modify it under the
 * terms of the GNU Library General Public Licence as published by the
 * Free Software Foundation; either version 2 of the Licence, or (at your
 * option) any later version.
 *
 * RepRap is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
 * Licence for more details.
 *
 * For this purpose the words "software" and "library" in the GNU Library
 * General Public Licence are taken to mean any and all computer programs
 * computer files data results documents and other copyright information
 * available from the RepRap project.
 *
 * You should have received a copy of the GNU Library General Public
 * Licence along with RepRap (in reports, it will be one of the
 * appendices, for example); if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA, or see
 *
 * http://www.gnu.org/
 *
 */

/*
 * This program listens to the serial port at 19200-8-N-1.
 * If you send it a number '1' through '9', it will blink
 * the LED that many times.  All other characters are 
 * ignored.  This is intended to test that the recieve 
 * wiring is correct when the transmit wiring may not 
 * be working
 */

#include <pic14.h>

typedef unsigned int config;
config at 0x2007 __CONFIG = _CP_OFF &
 _WDT_OFF &
 _BODEN_OFF &
 _PWRTE_ON &
 _INTRC_OSC_NOCLKOUT &
 _MCLRE_OFF &
 _LVP_OFF;

// baud settings with BRGH=1, all within 0.2%
const unsigned char spbrg[] = { 207, 103, 51, 25, 12 };
const char speeds[][6] = {"1200", "2400", "4800", "9600", "19200"};
const char* prefix = "Transmitting at ";
const char* baud = " baud ";
const char* correct = "(correct)\r\n";
const char* incorrect = "(ERROR)\r\n";
char* c;
unsigned char rate = 0;
unsigned char count;

#define LED PORTA4


void main() {
  unsigned char c;
  PORTA = 0x10; // RA4 is high to turn of LED
  CMCON = 0x07; // disable comparators
  TRISA = 0x00;

  PORTB = 0x00;
  TRISB = 0x02; // RB2 is RX

  RCSTA = BIN(00010000);
  SYNC = 0;
  BRGH = 1;
  SPBRG = 12; // 19200 at 4MHz
  SPEN = 1;

  while (1) {
    CREN = 1;
    while (!RCIF);
    c = RCREG;
    CREN = 0;
    if (c > '0' && c <= '9') {
      c -= '0';
      // set up tmr1 for 1.9Hz to control our blink
      T1CON = BIN(00110000);
      TMR1H = 0;
      TMR1L = 0;
      TMR1IF = 0;
      TMR1ON = 1;
      c <<= 1;
      while (c--) {
        while (!TMR1IF);
        LED = !LED;
        TMR1IF = 0;
      }
      TMR1ON = 0;
    }
  }
}

