/*
*
* RepRap, The Replicating Rapid Prototyper Project
*
* http://reprap.org/
*
* RepRap is copyright (C) 2005-6 University of Bath, the RepRap
* researchers (see the project's People webpage), and other contributors.
*
* RepRap is free; you can redistribute it and/or modify it under the
* terms of the GNU Library General Public Licence as published by the
* Free Software Foundation; either version 2 of the Licence, or (at your
* option) any later version.
*
* RepRap is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
* Licence for more details.
*
* For this purpose the words "software" and "library" in the GNU Library
* General Public Licence are taken to mean any and all computer programs
* computer files data results documents and other copyright information
* available from the RepRap project.
*
* You should have received a copy of the GNU Library General Public
* Licence along with RepRap (in reports, it will be one of the
* appendices, for example); if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA, or see
*
* http://www.gnu.org/
*
*/

/*
 * This program sends an endless stream of messages like:
 *   "Transmitting at 19200 baud (correct)"
 * to its serial port at several baud rates.  This is intended
 * to verify that your transmit wiring is correct, and your
 * baud rate settings are correct
 */
#include <pic14.h>

typedef unsigned int config;
config at 0x2007 __CONFIG = _CP_OFF &
                            _WDT_OFF &
                            _BODEN_OFF &
                            _PWRTE_ON &
                            _INTRC_OSC_NOCLKOUT &
                            _MCLRE_OFF &
                            _LVP_OFF;

// baud settings with BRGH=1, all within 0.2%
#define NUM_RATES 5
#define CORRECT_RATE 4
const unsigned char spbrg[] = { 207, 103, 51, 25, 12 };
const char speeds[][6] = {"1200", "2400", "4800", "9600", "19200"};
const char* prefix = "Transmitting at ";
const char* baud = " baud ";
const char* correct = "(correct)\r\n";
const char* incorrect = "(ERROR)\r\n";
char* c;
unsigned char rate = 0;
unsigned char count;

#define LED PORTA4


void main() {

  PORTA = 0x10; // RA4 is high to turn of LED
  CMCON = 0x07; // disable comparators
  TRISA = 0x00;

  PORTB = 0x00;
  TRISB = 0x02; // RB2 is RX

  RCSTA = BIN(00010000);
  /* TXSTA.*/
  SYNC = 0;
  /* TXSTA. */
  BRGH = 1;

  while (1) {
    for (rate = 0; rate < NUM_RATES; rate++) {
      SPBRG = spbrg[rate];
      SPEN = 1;
      TXEN = 1;
      for (count = 0; count < 3; ++count) {
        c = prefix;
        while (*c) {
          while (!TXIF)
            ;
          TXREG = *c++;
        }
        c = speeds[rate];
        while (*c) {
          while (!TXIF)
            ;
          TXREG = *c++;
        }

        c = baud;
        while (*c) {
          while (!TXIF)
            ;
          TXREG = *c++;
        }
        c = rate == CORRECT_RATE ? correct : incorrect;
        while (*c) {
          while (!TXIF)
            ;
          TXREG = *c++;
        }
      }
      while (!TRMT)
        ;
      TXEN = 0;
      SPEN = 0;
    }
    LED = !LED;
  }
}

